@@ -42,6 +42,7 @@ from google.cloud.dataflow_v1beta3.types import snapshots
 from .transports.base import DEFAULT_CLIENT_INFO, SnapshotsV1Beta3Transport
 from .transports.grpc import SnapshotsV1Beta3GrpcTransport
 from .transports.grpc_asyncio import SnapshotsV1Beta3GrpcAsyncIOTransport
+from .transports.rest import SnapshotsV1Beta3RestTransport
 
 
 class SnapshotsV1Beta3ClientMeta(type):
@@ -57,6 +58,7 @@ class SnapshotsV1Beta3ClientMeta(type):
     )  # type: Dict[str, Type[SnapshotsV1Beta3Transport]]
     _transport_registry["grpc"] = SnapshotsV1Beta3GrpcTransport
     _transport_registry["grpc_asyncio"] = SnapshotsV1Beta3GrpcAsyncIOTransport
+    _transport_registry["rest"] = SnapshotsV1Beta3RestTransport
 
     def get_transport_class(
         cls,
@@ -330,6 +332,9 @@ class SnapshotsV1Beta3Client(metaclass=SnapshotsV1Beta3ClientMeta):
             transport (Union[str, SnapshotsV1Beta3Transport]): The
                 transport to use. If set to None, a transport is chosen
                 automatically.
+                NOTE: "rest" transport functionality is currently in a
+                beta state (preview). We welcome your feedback via an
+                issue in this library's source repository.
             client_options (google.api_core.client_options.ClientOptions): Custom options for the
                 client. It won't take effect if a ``transport`` instance is provided.
                 (1) The ``api_endpoint`` property can be used to override the
@@ -422,6 +427,13 @@ class SnapshotsV1Beta3Client(metaclass=SnapshotsV1Beta3ClientMeta):
 
         .. code-block:: python
 
+            # This snippet has been automatically generated and should be regarded as a
+            # code template only.
+            # It will require modifications to work:
+            # - It may require correct/in-range values for request initialization.
+            # - It may require specifying regional endpoints when creating the service
+            #   client as shown in:
+            #   https://googleapis.dev/python/google-api-core/latest/client_options.html
             from google.cloud import dataflow_v1beta3
 
             def sample_get_snapshot():
@@ -499,6 +511,13 @@ class SnapshotsV1Beta3Client(metaclass=SnapshotsV1Beta3ClientMeta):
 
         .. code-block:: python
 
+            # This snippet has been automatically generated and should be regarded as a
+            # code template only.
+            # It will require modifications to work:
+            # - It may require correct/in-range values for request initialization.
+            # - It may require specifying regional endpoints when creating the service
+            #   client as shown in:
+            #   https://googleapis.dev/python/google-api-core/latest/client_options.html
             from google.cloud import dataflow_v1beta3
 
             def sample_delete_snapshot():
@@ -575,6 +594,13 @@ class SnapshotsV1Beta3Client(metaclass=SnapshotsV1Beta3ClientMeta):
 
         .. code-block:: python
 
+            # This snippet has been automatically generated and should be regarded as a
+            # code template only.
+            # It will require modifications to work:
+            # - It may require correct/in-range values for request initialization.
+            # - It may require specifying regional endpoints when creating the service
+            #   client as shown in:
+            #   https://googleapis.dev/python/google-api-core/latest/client_options.html
             from google.cloud import dataflow_v1beta3
 
             def sample_list_snapshots():
